@@ -38,6 +38,8 @@
 #include <iostream>
 #include <sstream>
 #include <cstdlib>
+#include <random>
+#include <chrono>
 #include <wx/version.h>
 #include <wx/filename.h>
 #include <wx/stdpaths.h>
@@ -106,9 +108,6 @@ MainFrame::MainFrame(const wxString& title, const wxSize& size = wxDefaultSize,
 	bitmap_dice[4] = new wxBitmap(dice::five_xpm);
 	bitmap_dice[5] = new wxBitmap(dice::six_xpm);
 	
-	//randomize the random-number generator based on the time
-	srand( (unsigned)time( NULL ) );
-	
 	AddMenus();
 
 	AddControlsAndLayout();
@@ -329,10 +328,33 @@ void MainFrame::AddControlsAndLayout()
 	topSizer->Add(sectionsSizer);
 	topSizer->Add(diceSizer);	
 
-	panel->SetSizer(topSizer);
-	
-	topSizer->Fit(this);
-	topSizer->SetSizeHints(this);
+	AdjustFrameSize(topSizer);
+
+#if (wxMAJOR_VERSION == 2 && wxMINOR_VERSION == 8 && defined __WXGTK__)
+	if (!m_menuBarHeight) {
+		// This checks for the bug in Ubuntu with wxGTK-2.8 where the
+		// menu-bar only appears after OnInternalIdle() event has been
+		// called. This results in wrong size calculation for the
+		// window, so we reserve additional space for the menu bar so
+		// when it appears it won't push the bottom of the window
+		// outside the boundary.
+		wxSize suggested_window_size = ClientToWindowSize(topSizer->GetMinSize());
+		suggested_window_size.SetHeight((GetMenuBar()->GetSize()).GetHeight() +
+			suggested_window_size.GetHeight());
+		this->SetSize(suggested_window_size);
+		this->SetSizeHints(suggested_window_size, suggested_window_size);
+	}
+#endif
+}
+
+void MainFrame::AdjustFrameSize(wxSizer *sizer)
+{
+	auto panel = (wxPanel*)FindWindow(ID_PANEL);
+
+	panel->SetSizerAndFit(sizer);
+	wxSize frame_size = ClientToWindowSize(panel->GetSize());
+	this->SetSizeHints(frame_size, frame_size);
+	sizer->Fit(this);
 }
 
 /**
@@ -465,7 +487,7 @@ void MainFrame::OnUndo(wxCommandEvent& event)
 		((wxButton*) FindWindow(ID_ROLL)) -> Enable(false);
 	
 	// change the displayed roll counter
-	wxString caption = wxString::Format(wxT("Roll! (%i)"),m_rolls);
+	wxString caption = wxString::Format(wxT("Roll! (%hi)"),m_rolls);
 	FindWindow(ID_ROLL)->SetLabel(caption);
 
 	//restore the 'keep' checkboxes
@@ -485,13 +507,10 @@ void MainFrame::OnUndo(wxCommandEvent& event)
 
 	//undo also the yahtzee bonus if needed
 	if (m_yahtzeebonus) {
-		long temp;
 		wxString tempstr;
 	
 		tempstr = ((wxTextCtrl*) FindWindow(ID_YAHTZEEBONUSTEXT)) -> GetValue();
-		tempstr.ToLong(&temp,10);
-		temp -= 100; //this line reduces the points given for the yahtzee bonus
-		tempstr.Printf(wxT("%i"),temp);
+		tempstr.Printf(wxT("%li"), wxStrtol(tempstr, NULL, 10) - 100);
 		((wxTextCtrl*) FindWindow(ID_YAHTZEEBONUSTEXT)) -> SetValue(tempstr);
 	}
 
@@ -567,45 +586,45 @@ void MainFrame::OnSettings( wxCommandEvent& event)
 void MainFrame::OnRollButton ()
 {
 	short int dice[5];	//holds the dices score
+
+	static std::mt19937::result_type seed = std::chrono::high_resolution_clock::now().time_since_epoch().count();
+	static auto dice_rand = std::bind(std::uniform_int_distribution<int>(1,6),
+				          std::mt19937(seed));
+	static auto num_throws_rand = std::bind(std::uniform_int_distribution<int>(5,18),
+					        std::mt19937(seed * 0x101010101));
 	
 	// If this is the first roll of the game, notify the statistics
 	// object
 	if (m_numofplaysleft == 13 && m_rolls == 3)
 		m_stats->game_started();
 
-	//fill the dice array with the old values
-	dice[0]=m_score_dice.GetDice(1);
-	dice[1]=m_score_dice.GetDice(2);
-	dice[2]=m_score_dice.GetDice(3);
-	dice[3]=m_score_dice.GetDice(4);
-	dice[4]=m_score_dice.GetDice(5);
-	//roll the dice...
-	if (m_config->get("dice-animation")=="True") {
-		int dice_throws[5] = {0,0,0,0,0};
-		for (int i=0; i<5; i++) { //set the number of rolls for each dice
-			if (!((wxCheckBox*) FindWindow(i + ID_DICE1KEEP))->IsChecked()) {
-				dice_throws[i] = (rand()%15)+3; //ensures the number is at least one.
+	// fill the dice array with the old values
+	int dice_throws[5];
+	for (int i = 0 ; i < 5; ++i) {
+		dice[i]=m_score_dice.GetDice(i+1);
+		dice_throws[i] = 0;
+		if ( ((wxCheckBox*) FindWindow(i + ID_DICE1KEEP))->IsChecked() ) {
+			dice_throws[i] = 0;
+		} else {
+			dice_throws[i] = 1;
+			if (m_config->get("dice-animation") == "True") {
+				dice_throws[i] = num_throws_rand();
 			}
 		}
-		while (dice_throws[0] || dice_throws[1] || dice_throws[2] || dice_throws[3] || dice_throws[4]) {
-			for (int i=0 ; i<5; i++){
-				if(dice_throws[i]){
-					dice_throws[i]--;
-					dice[i] = (int)(6.0*rand()/RAND_MAX)+1;
-					((wxDynamicBitmap*) FindWindow(i + 
-						ID_DICE1)) -> SetBitmap(
-						bitmap_dice[dice[i]-1]);
-				}
+	}
+
+	while (dice_throws[0] || dice_throws[1] || dice_throws[2] || dice_throws[3] || dice_throws[4]) {
+		for (int i=0 ; i<5; i++){
+			if(dice_throws[i]){
+				dice_throws[i]--;
+				dice[i] = dice_rand();
+				((wxDynamicBitmap*) FindWindow(i + 
+					ID_DICE1)) -> SetBitmap(
+					bitmap_dice[dice[i]-1]);
 			}
-			::wxMilliSleep(100);
 		}
-	} else {
-		for (int i=0; i<5; i++) {
-			if (!((wxCheckBox*) FindWindow(i + ID_DICE1KEEP))->IsChecked()) {
-				dice[i] = (int)(6.0*rand()/RAND_MAX)+1;
-				((wxDynamicBitmap*) FindWindow(i + ID_DICE1))->
-					SetBitmap(bitmap_dice[dice[i]-1]);
-			}
+		if (m_config->get("dice-animation")=="True") {
+			wxMilliSleep(100);
 		}
 	}
 
@@ -614,7 +633,7 @@ void MainFrame::OnRollButton ()
 	--m_rolls;
 	
 	// change the displayed roll counter
-	wxString caption = wxString::Format(wxT("Roll! (%i)"),m_rolls);
+	wxString caption = wxString::Format(wxT("Roll! (%hi)"),m_rolls);
 	FindWindow(ID_ROLL)->SetLabel(caption);
 
 	#ifndef DEBUG
@@ -696,7 +715,7 @@ void MainFrame::OnUpperButtons (wxCommandEvent& event)
 		break;
 	}
 	
-	out.Printf(wxT("%i"),temp);
+	out.Printf(wxT("%hi"),temp);
 	wxTextCtrl *text_ctrl = dynamic_cast<wxTextCtrl*>(FindWindow(event.GetId() - ID_ACES + ID_ACESTEXT));
 	text_ctrl->SetValue(out);
 	text_ctrl->SetBackgroundColour(*wxWHITE);
@@ -717,7 +736,7 @@ void MainFrame::On3ofakindButton(wxCommandEvent& event)
 	YahtzeeBonus();
 	wxString out;
 	
-	out.Printf(wxT("%i"),m_score_dice.ThreeOfAKind());
+	out.Printf(wxT("%hi"),m_score_dice.ThreeOfAKind());
 	wxTextCtrl *text_ctrl = dynamic_cast<wxTextCtrl*>(FindWindow(ID_THREEOFAKINDTEXT));
 	text_ctrl->SetValue(out);
 	text_ctrl->SetBackgroundColour(*wxWHITE);
@@ -738,7 +757,7 @@ void MainFrame::On4ofakindButton(wxCommandEvent& event)
 	YahtzeeBonus();
 	wxString out;
 
-	out.Printf(wxT("%i"),m_score_dice.FourOfAKind());
+	out.Printf(wxT("%hi"),m_score_dice.FourOfAKind());
 	wxTextCtrl *text_ctrl = dynamic_cast<wxTextCtrl*>(FindWindow(ID_FOUROFAKINDTEXT));
 	text_ctrl->SetValue(out);
 	text_ctrl->SetBackgroundColour(*wxWHITE);
@@ -760,7 +779,7 @@ void MainFrame::OnFullHouseButton(wxCommandEvent& event)
 	
 	YahtzeeBonus();
 
-	out.Printf(wxT("%i"), m_score_dice.FullHouse());
+	out.Printf(wxT("%hi"), m_score_dice.FullHouse());
 	wxTextCtrl *text_ctrl = dynamic_cast<wxTextCtrl*>(FindWindow(ID_FULLHOUSETEXT));
 	text_ctrl->SetValue(out);
 	text_ctrl->SetBackgroundColour(*wxWHITE);
@@ -782,7 +801,7 @@ void MainFrame::OnSmallSequenceButton(wxCommandEvent& event)
 
 	YahtzeeBonus();
 	
-	out.Printf(wxT("%i"), m_score_dice.SmallSequence());
+	out.Printf(wxT("%hi"), m_score_dice.SmallSequence());
 	wxTextCtrl *text_ctrl = dynamic_cast<wxTextCtrl*>(FindWindow(ID_SMALLSEQUENCETEXT));
 	text_ctrl->SetValue(out);
 	text_ctrl->SetBackgroundColour(*wxWHITE);
@@ -804,7 +823,7 @@ void MainFrame::OnLargeSequenceButton(wxCommandEvent& event)
 
 	YahtzeeBonus();
 
-	out.Printf(wxT("%i"), m_score_dice.LargeSequence());
+	out.Printf(wxT("%hi"), m_score_dice.LargeSequence());
 	wxTextCtrl *text_ctrl = dynamic_cast<wxTextCtrl*>(FindWindow(ID_LARGESEQUENCETEXT));
 	text_ctrl->SetValue(out);
 	text_ctrl->SetBackgroundColour(*wxWHITE);
@@ -826,7 +845,7 @@ void MainFrame::OnYahtzeeButton(wxCommandEvent& event)
 		return;
 	
 	if (m_score_dice.IsYahtzee()) m_yahtzee = true;
-	out.Printf(wxT("%i"), m_score_dice.Yahtzee());
+	out.Printf(wxT("%hi"), m_score_dice.Yahtzee());
 	wxTextCtrl *text_ctrl = dynamic_cast<wxTextCtrl*>(FindWindow(ID_YAHTZEETEXT));
 	text_ctrl->SetValue(out);
 	text_ctrl->SetBackgroundColour(*wxWHITE);
@@ -847,7 +866,7 @@ void MainFrame::OnChanceButton (wxCommandEvent& event)
 
 	YahtzeeBonus();
 
-	out.Printf(wxT("%i"),m_score_dice.Chance());
+	out.Printf(wxT("%hi"),m_score_dice.Chance());
 	wxTextCtrl *text_ctrl = dynamic_cast<wxTextCtrl*>(FindWindow(ID_CHANCETEXT));
 	text_ctrl->SetValue(out);
 	text_ctrl->SetBackgroundColour(*wxWHITE);
@@ -910,7 +929,6 @@ void MainFrame::ResetRolls()
  */
 void MainFrame::YahtzeeBonus()
 {
-	long temp;
 	wxString tempstr;
 	
 	//if the player didn't have any yathzees yet he can't have the bonus;
@@ -918,9 +936,7 @@ void MainFrame::YahtzeeBonus()
 		return;
 	if (m_score_dice.IsYahtzee()) {
 		tempstr = ((wxTextCtrl*) FindWindow(ID_YAHTZEEBONUSTEXT)) -> GetValue();
-		tempstr.ToLong(&temp,10);
-		temp += 100;
-		tempstr.Printf(wxT("%i"),temp);
+		tempstr.Printf(wxT("%li"), wxStrtol(tempstr, NULL, 10) + 100);
 		((wxTextCtrl*) FindWindow(ID_YAHTZEEBONUSTEXT)) -> SetValue(tempstr);
 		m_yahtzeebonus = true;
 	}	
@@ -952,17 +968,15 @@ void MainFrame::EndofGame()
 		return;
 	
 	wxString tempstr;
-	long temp;
 	long upperscore = 0;
 	long lowerscore = 0;
 
 	for (int i = ID_ACESTEXT; i<=ID_SIXESTEXT; i++){
 		tempstr = ((wxTextCtrl*) FindWindow(i)) -> GetValue();
-		tempstr.ToLong(&temp,10);
-		upperscore +=temp;
+		upperscore += wxStrtol(tempstr, NULL, 10);
 	}
 	
-	tempstr.Printf(wxT("%i"),upperscore);
+	tempstr.Printf(wxT("%li"),upperscore);
 	((wxTextCtrl*) FindWindow(ID_UPPERSECTIONTOTAL)) -> SetValue(tempstr);
 	
 	//check for bonus
@@ -972,25 +986,24 @@ void MainFrame::EndofGame()
 	} else
 		((wxTextCtrl*) FindWindow(ID_BONUS)) -> SetValue(wxT("0"));
 	
-	tempstr.Printf(wxT("%i"),upperscore);
+	tempstr.Printf(wxT("%li"),upperscore);
 	((wxTextCtrl*) FindWindow(ID_UPPERTOTAL)) -> SetValue(tempstr);
 	
 	//calculate total on lower section
 	for (int i = ID_THREEOFAKINDTEXT; i<=ID_YAHTZEEBONUSTEXT; i++) {
 		tempstr = ((wxTextCtrl*) FindWindow(i)) -> GetValue();
-		tempstr.ToLong(&temp,10);
-		lowerscore +=temp;
+		lowerscore += wxStrtol(tempstr, NULL, 10);
 	}
 	
-	tempstr.Printf(wxT("%i"),lowerscore);
+	tempstr.Printf(wxT("%li"),lowerscore);
 	((wxTextCtrl*) FindWindow(ID_LOWERTOTAL)) -> SetValue(tempstr);
-	tempstr.Printf(wxT("%i"),upperscore + lowerscore);
+	tempstr.Printf(wxT("%li"),upperscore + lowerscore);
 	((wxTextCtrl*) FindWindow(ID_GRANDTOTAL)) -> SetValue(tempstr);
 
 	//disable the roll button;
 	((wxButton*) FindWindow(ID_ROLL)) -> Enable(false);
 
-	tempstr.Printf(wxT("Your final score is %i points!"),lowerscore+upperscore);
+	tempstr.Printf(wxT("Your final score is %li points!"),lowerscore+upperscore);
 	wxMessageBox(tempstr, wxT("Game Ended"), wxOK | wxICON_INFORMATION, this);
 
 	m_stats->game_finished(lowerscore+upperscore);
@@ -1075,16 +1088,13 @@ void MainFrame::CalculateSubTotal()
 	long upperscore = 0;
 	long lowerscore = 0;
 	wxString tempstr;
-	long temp;
-
 
 	for (int i = ID_ACESTEXT; i<=ID_SIXESTEXT; i++){
 		tempstr = ((wxTextCtrl*) FindWindow(i)) -> GetValue();
-		tempstr.ToLong(&temp,10);
-		upperscore +=temp;
+		upperscore += wxStrtol(tempstr, NULL, 10);
 	}
 	
-	tempstr.Printf(wxT("%i"),upperscore);
+	tempstr.Printf(wxT("%li"),upperscore);
 	((wxTextCtrl*) FindWindow(ID_UPPERSECTIONTOTAL)) -> SetValue(tempstr);
 	if (upperscore >= 63) {
 		((wxTextCtrl*) FindWindow(ID_BONUS))->SetValue(wxT("35"));
@@ -1095,11 +1105,10 @@ void MainFrame::CalculateSubTotal()
 
 	for (int i = ID_THREEOFAKINDTEXT; i<=ID_YAHTZEEBONUSTEXT; i++) {
 		tempstr = ((wxTextCtrl*) FindWindow(i)) -> GetValue();
-		tempstr.ToLong(&temp,10);
-		lowerscore +=temp;
+		lowerscore += wxStrtol(tempstr, NULL, 10);
 	}
 	
-	tempstr.Printf(wxT("%i"),lowerscore);
+	tempstr.Printf(wxT("%li"),lowerscore);
 	((wxTextCtrl*) FindWindow(ID_LOWERTOTAL)) -> SetValue(tempstr);
 }
 
@@ -1110,10 +1119,10 @@ void MainFrame::Relayout()
 	wxFlexGridSizer *diceSizer;
 	bool roll_button_enabled;
 
-	lowersection->GetStaticBox()->Destroy();
-	uppersection->GetStaticBox()->Destroy();
 	sectionsSizer->Remove(lowersection);
+	lowersection = NULL;
 	sectionsSizer->Remove(uppersection);
+	uppersection = NULL;
 
 	if (m_config->get("horizontal-layout")=="True") {
 		topSizer = new wxBoxSizer( wxVERTICAL );
@@ -1180,7 +1189,7 @@ void MainFrame::Relayout()
 	sectionsSizer->Add(uppersection,0,wxALL,5);
 	sectionsSizer->Add(lowersection,0,wxALL,5);
 
-	//Change the roll button size if we need to
+	//Change the roll button size if we need trameo
 	roll_button_enabled = FindWindow(ID_ROLL)->IsEnabled();
 	if (m_config->get("horizontal-layout")=="True") {
 		FindWindow(ID_ROLL)->Destroy();
@@ -1193,7 +1202,7 @@ void MainFrame::Relayout()
 
 	// if there are rolls left we should display the count of them
 	if (roll_button_enabled) {
-		wxString caption = wxString::Format(wxT("Roll! (%i)"),m_rolls);
+		wxString caption = wxString::Format(wxT("Roll! (%hi)"),m_rolls);
 		FindWindow(ID_ROLL)->SetLabel(caption);
 	}
 
@@ -1235,13 +1244,7 @@ void MainFrame::Relayout()
 	topSizer->Add(sectionsSizer);
 	topSizer->Add(diceSizer);	
 	
-	topSizer->SetSizeHints(this);
-
-	topSizer->Layout();
-	FindWindow(ID_PANEL)->SetSizerAndFit(topSizer);
-	
-	topSizer->Fit(this);
-	
+	AdjustFrameSize(topSizer);
 }
 
 /**
@@ -1276,43 +1279,43 @@ void MainFrameEvtHandler::OnScoreMouseEnter (wxMouseEvent& event)
 	text_control = ((wxTextCtrl*)m_main_frame->FindWindow(id-ID_ACES+ID_ACESTEXT));
 	switch (id) {
 	case ID_ACES:
-		out.Printf(wxT("%i"), m_main_frame->m_score_dice.Aces());
+		out.Printf(wxT("%hi"), m_main_frame->m_score_dice.Aces());
 		break;
 	case ID_TWOS:
-		out.Printf(wxT("%i"), m_main_frame->m_score_dice.Twos());
+		out.Printf(wxT("%hi"), m_main_frame->m_score_dice.Twos());
 		break;
 	case ID_THREES:
-		out.Printf(wxT("%i"), m_main_frame->m_score_dice.Threes());
+		out.Printf(wxT("%hi"), m_main_frame->m_score_dice.Threes());
 		break;
 	case ID_FOURS:
-		out.Printf(wxT("%i"), m_main_frame->m_score_dice.Fours());
+		out.Printf(wxT("%hi"), m_main_frame->m_score_dice.Fours());
 		break;
 	case ID_FIVES:
-		out.Printf(wxT("%i"), m_main_frame->m_score_dice.Fives());
+		out.Printf(wxT("%hi"), m_main_frame->m_score_dice.Fives());
 		break;
 	case ID_SIXES:
-		out.Printf(wxT("%i"), m_main_frame->m_score_dice.Sixes());
+		out.Printf(wxT("%hi"), m_main_frame->m_score_dice.Sixes());
 		break;
 	case ID_THREEOFAKIND:
-		out.Printf(wxT("%i"), m_main_frame->m_score_dice.ThreeOfAKind());
+		out.Printf(wxT("%hi"), m_main_frame->m_score_dice.ThreeOfAKind());
 		break;
 	case ID_FOUROFAKIND:
-		out.Printf(wxT("%i"), m_main_frame->m_score_dice.FourOfAKind());
+		out.Printf(wxT("%hi"), m_main_frame->m_score_dice.FourOfAKind());
 		break;
 	case ID_FULLHOUSE:
-		out.Printf(wxT("%i"), m_main_frame->m_score_dice.FullHouse());
+		out.Printf(wxT("%hi"), m_main_frame->m_score_dice.FullHouse());
 		break;
 	case ID_SMALLSEQUENCE:
-		out.Printf(wxT("%i"), m_main_frame->m_score_dice.SmallSequence());
+		out.Printf(wxT("%hi"), m_main_frame->m_score_dice.SmallSequence());
 		break;
 	case ID_LARGESEQUENCE:
-		out.Printf(wxT("%i"), m_main_frame->m_score_dice.LargeSequence());
+		out.Printf(wxT("%hi"), m_main_frame->m_score_dice.LargeSequence());
 		break;
 	case ID_YAHTZEE:
-		out.Printf(wxT("%i"), m_main_frame->m_score_dice.Yahtzee());
+		out.Printf(wxT("%hi"), m_main_frame->m_score_dice.Yahtzee());
 		break;
 	case ID_CHANCE:
-		out.Printf(wxT("%i"), m_main_frame->m_score_dice.Chance());
+		out.Printf(wxT("%hi"), m_main_frame->m_score_dice.Chance());
 		break;
 	}
 	text_control->SetValue(out); 		
