@@ -57,7 +57,7 @@ bool MyApp::OnInit()
 	::wxInitAllImageHandlers();
 
 	// Create the main application window
-	main_frame::MainFrame *frame = new main_frame::MainFrame(wxT("Open Yahtzee"), wxDefaultSize, wxDEFAULT_FRAME_STYLE & (~wxRESIZE_BORDER));
+	main_frame::MainFrame *frame = new main_frame::MainFrame(wxT("Open Yahtzee"), wxDefaultSize, wxDEFAULT_FRAME_STYLE);
 	
 	//Center the frame and show it
 	frame->Centre(true);
